import { Person } from 'app/shared/model/person.model';
import { SearchResultDTO } from 'app/shared/model/search/search-result-dto.model';

export enum IExerciseType {
  COLLECTION = 'collection',
  PROGRAMMING_EXERCISE = 'programming exercise',
  EXERCISE = 'exercise',
  OTHER = 'other',
}

export enum ILanguages {
  GERMAN = 'de',
  ENGLISH = 'en',
}

// just a cheap trick, to make enum available to HTML.
// see https://stackoverflow.com/questions/35835984/how-to-use-a-typescript-enum-value-in-an-angular2-ngswitch-statement
// export function IExerciseTypeAware(constructor: Function) {
//    constructor.prototype.IExerciseType = IExerciseType;
// }

// export class IExerciseTypeClass {
//     myEnum: typeof IExerciseType;
//     myEnumField: IExerciseType;
// }

/** @deprecated unnecessary wrapper around exercises. Only causes extra effort. On long term should be replaced by SearchResultDTO  */
export interface Exercise {
  // from metadata (required)

  title: string;
  license: string;

  // from metadata (optional)
  description: string;
  programmingLanguage: string[];
  language: string[];
  creator: Person[];
  publisher: Person[];
  contributor: Person[];
  requires: string[];
  imageURL: string;
  timeRequired: string;
  deprecated: boolean;
  difficulty?: string;
  educationLevel: string;
  format: Array<string>;
  keyword: Array<string>;
  // language: Array<string>;
  // repositoryURL: string;
  // source: Array<string>;
  status: string;
  // structure: string;
  structure: string;
  version: string;
  metadataVersion: string;

  gitlabURL: string;

  // not in metadata
  rating: number;
  lastUpdate: Date;
  originalResult: SearchResultDTO;

  // thesis
  views: number;
  downloads: number;
  numberOfLikes: number;
  userHasLiked: boolean;
}

export function searchResultToExercise(searchResult: SearchResultDTO): Exercise {
  return {
    title: searchResult.metadata.title,
    license: searchResult.metadata.license,
    description: searchResult.metadata.description,
    programmingLanguage: searchResult.metadata.programmingLanguage,
    language: searchResult.metadata.language,
    creator: searchResult.metadata.creator,
    contributor: searchResult.metadata.contributor,
    publisher: searchResult.metadata.publisher,
    imageURL: searchResult.metadata.image,
    rating: searchResult.ranking5,
    lastUpdate: searchResult.project.last_activity_at,
    timeRequired: searchResult.metadata.timeRequired,
    requires: searchResult.metadata.requires,
    deprecated: searchResult.metadata.deprecated,
    difficulty: searchResult.metadata.difficulty,
    educationLevel: searchResult.metadata.educationLevel,
    format: searchResult.metadata.format,
    keyword: searchResult.metadata.keyword,
    status: searchResult.metadata.status,
    structure: searchResult.metadata.structure,
    version: searchResult.metadata.version,
    metadataVersion: searchResult.metadata.metadataVersion,

    gitlabURL: searchResult.project.url,
    originalResult: searchResult,
    views: searchResult.views,
    downloads: searchResult.downloads,

    numberOfLikes: 0,
    userHasLiked: false,
  };
}
